/***********************************************************************
 *
 * Copyright (C) 2005 by FS Forth-Systeme GmbH
 * All rights reserved.
 *
 * $Id: cc9p9360.c,v 1.3 2006-08-09 10:36:27 uzeisberger Exp $
 * @Author: Jonas Dietsche
 * @References: [1] NS9360 Hardware Reference
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***********************************************************************/
/***********************************************************************
 *  @History:
 *     2005/11/29 : moved to kernel 2.6.14
 *     2005/09/12 : initial version
 ***********************************************************************/
#include "ns9360.h"	/*include the CPU specific stuff */

#ifdef CONFIG_A9M9750DEV_BOARD
# include <linux/serial_8250.h>
#endif /*CONFIG_A9M9750DEV_BOARD*/

#ifdef CONFIG_A9M9750DEV_BOARD
static struct plat_serial8250_port serial_platform_data[] = {
	{
		.iobase		= BASE_EXT_UARTA,
		.membase	= (unsigned char*)BASE_EXT_UARTA,
		.mapbase	= BASE_EXT_UARTA,
		.irq		= IRQ_FPGA_UARTA,
		.iotype		= UPIO_MEM,
		.uartclk	= 18432000,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF | UPF_SHARE_IRQ,
	},
	{
		.iobase		= BASE_EXT_UARTB,
		.membase	= (unsigned char*)BASE_EXT_UARTB,
		.mapbase	= BASE_EXT_UARTB,
		.irq		= IRQ_FPGA_UARTB,
		.iotype		= UPIO_MEM,
		.uartclk	= 18432000,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF | UPF_SHARE_IRQ,
	},
	{
		.iobase		= BASE_EXT_UARTC,
		.membase	= (unsigned char*)BASE_EXT_UARTC,
		.mapbase	= BASE_EXT_UARTC,
		.irq		= IRQ_FPGA_UARTC,
		.iotype		= UPIO_MEM,
		.uartclk	= 18432000,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF | UPF_SHARE_IRQ,
	},
	{
		.iobase		= BASE_EXT_UARTD,
		.membase	= (unsigned char*)BASE_EXT_UARTD,
		.mapbase	= BASE_EXT_UARTD,
		.irq		= IRQ_FPGA_UARTD,
		.iotype		= UPIO_MEM,
		.uartclk	= 18432000,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF | UPF_SHARE_IRQ,
	},
	{ },
};

static struct platform_device serial_device = {
	.name			= "serial8250",
	.id			= 0,
	.dev			= {
		.platform_data	= serial_platform_data,
	},
};
#endif /*CONFIG_A9M9750DEV_BOARD*/

#if defined(CONFIG_A9M9XX0_CF) || defined(CONFIG_A9M9XX0_CF_MODULE)
static struct a9m9xx0_cf_data a9m9xx0_cf_cfg __initdata = {
	.irq = -1,
	.detect = -1,
	.reset = -1,
};
#endif /*defined(CONFIG_A9M9XX0_CF) || defined(CONFIG_A9M9XX0_CF_MODULE) */

#if defined(CONFIG_TOUCHSCREEN_A9M9XX0) || defined(CONFIG_TOUCHSCREEN_A9M9XX0_MODULE)
static struct a9m9xx0_ts_data a9m9xx0_ts_cfg __initdata = {
	.samples_ms = 20,
	.samples_avg = 1,
	.port = 0,
	.bitrate = 200000,
	.y_top = 3740,
	.y_bottom = 225,
	.x_right = 210,
	.x_left = 3870,
};
#endif /*defined(CONFIG_TOUCHSCREEN_A9M9XX0) || defined(CONFIG_TOUCHSCREEN_A9M9XX0_MODULE)*/

void  __init ns_init( void )
{
	int i = 0;

	spin_lock_init( &ns_gpio_lock );
#ifdef CONFIG_A9M9750DEV_BOARD
	/* start setup CS0 */
 	writel( 0x40000000, NS_SYS_CS_STATIC_BASE( 0 ) );
  	rmwl( NS_SYS_CS_STATIC_MASK(0), | 0xFFFFF001 );

	/* Interrupt Level-sensitive */
	writel( NS_SYS_EXT_INT_CTRL_PLTY, NS_SYS_EXT_INT_CTRL(2) );

	writel( NS_MEM_STAT_CFG_MW_8 | NS_MEM_STAT_CFG_PB, NS_MEM_STAT_CFG( 0 ) );
	writel(0x2 , NS_MEM_STAT_WAIT_WEN( 0 ) );
	writel( 0x2, NS_MEM_STAT_WAIT_OEN( 0 ) );
	writel( 0x6, NS_MEM_STAT_RD( 0 ) );
	writel( 0x6, NS_MEM_STAT_WR( 0 ) );
	/* end setup CS0 */
	
	/* start setup CS2 */
 	writel( 0x60000000, NS_SYS_CS_STATIC_BASE( 2 ) );
  	rmwl( NS_SYS_CS_STATIC_MASK(0), | 0x6FFFF001 );

	writel( NS_MEM_STAT_CFG_MW_16 | NS_MEM_STAT_CFG_PB | NS_MEM_STAT_CFG_EW, NS_MEM_STAT_CFG( 2 ) );
	writel( 0x10, NS_MEM_STAT_EXT_WAIT );
	writel(0x0 , NS_MEM_STAT_WAIT_WEN( 2 ) );
	writel( 0x0, NS_MEM_STAT_WAIT_OEN( 2 ) );
	writel( 0x4, NS_MEM_STAT_RD( 2 ) );
	writel( 0x4, NS_MEM_STAT_WR( 2 ) );
	/* end setup CS2 */


	/*use IRQ2, GPIO11 */
	request_gpio( 11, 1, "serial");
	set_gpio_cfg( 11, NS_GPIO_CFG_FUNC_1 );

	platform_device_register(&serial_device);

#endif /*CONFIG_A9M9750DEV_BOARD*/
	while( resources[ i ].name != NULL ) {
		request_resource( &iomem_resource, &resources[ i++ ] );
	}
	platform_add_devices(usb_devices, 1);

#if defined(CONFIG_TOUCHSCREEN_A9M9XX0) || defined(CONFIG_TOUCHSCREEN_A9M9XX0_MODULE)
	set_a9m9xx0ts_data( &a9m9xx0_ts_cfg );
	platform_device_register( &a9m9xx0_device_ts );
#endif
#if defined(CONFIG_A9M9XX0_CF) || defined(CONFIG_A9M9XX0_CF_MODULE)
	set_a9m9xx0cf_data( &a9m9xx0_cf_cfg );
	platform_device_register( &a9m9xx0_devide_cf );
#endif
}


/* look in debug.S when adjusting BOOT_MEM */
MACHINE_START(A9M9360, "A9M9360")
	/* Maintainer: Jonas Dietsche <jonas_dietsche@digi.com> */
	.phys_io		= 0x90000000,
	.io_pg_offst	= ((0xf9000000) >> 18) & 0xfffc,
	.boot_params	= 0x00000100,
	.map_io		= ns_map_io,
	.init_irq		= ns_init_irq,
	.timer 		= &ns_timer,
	.init_machine	= ns_init,
MACHINE_END
