/*
 * arch/arm/mach-ns9xxx/leds.c
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/config.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/leds.h>
#include <asm/system.h>

#include <asm/arch/hardware.h>
#include <asm/arch/regs-bbus.h>

#include "leds.h"

#define JSDEV_LED_CPU_GPIO_PIN		48
#define JSDEV_LED_TIMMER_GPIO_PIN	49

#define LED_ON				0
#define LED_OFF				1



int jsdev_leds_config(void)
{
#ifdef CONFIG_LEDS_CPU
	/* Configure GPIO pins for led operations */
	if (ns9xxx_request_gpio(JSDEV_LED_CPU_GPIO_PIN, "CPU led") != 0) {
		printk( KERN_WARNING "leds-jsdev: failed to reserve gpio %d\n", JSDEV_LED_CPU_GPIO_PIN );
		return -EBUSY;
	}
	(void)ns9xxx_gpio_cfgpin(JSDEV_LED_CPU_GPIO_PIN, GPIO_CFG_OUTPUT);
#endif

#ifdef CONFIG_LEDS_TIMER
	if (( JSDEV_LED_CPU_GPIO_PIN != JSDEV_LED_TIMMER_GPIO_PIN ) &&
	    (ns9xxx_request_gpio(JSDEV_LED_TIMMER_GPIO_PIN, "Timmer led") != 0) ) {
		printk( KERN_WARNING "leds-jsdev: failed to reserve gpio %d\n", JSDEV_LED_TIMMER_GPIO_PIN );
		ns9xxx_release_gpio(JSDEV_LED_CPU_GPIO_PIN);
		return -EBUSY;
	}
	(void)ns9xxx_gpio_cfgpin(JSDEV_LED_TIMMER_GPIO_PIN, GPIO_CFG_OUTPUT);
#endif
	return 0;
}


void jsdev_leds_event(led_event_t evt)
{
	unsigned long flags;

	local_irq_save(flags);

	switch (evt) {
		case led_start:
			(void)ns9xxx_gpio_setpin(JSDEV_LED_CPU_GPIO_PIN, LED_ON);
			break;

		case led_stop:
			(void)ns9xxx_gpio_setpin(JSDEV_LED_CPU_GPIO_PIN, LED_OFF);
			break;

		case led_claim:
			break;

		case led_release:
			break;

#ifdef CONFIG_LEDS_TIMER
		case led_timer:
			if (ns9xxx_gpio_getpin(JSDEV_LED_TIMMER_GPIO_PIN))
				(void)ns9xxx_gpio_setpin(JSDEV_LED_TIMMER_GPIO_PIN, LED_ON);
			else
				(void)ns9xxx_gpio_setpin(JSDEV_LED_TIMMER_GPIO_PIN, LED_OFF);
			break;
#endif

#ifdef CONFIG_LEDS_CPU
		case led_idle_start:
			(void)ns9xxx_gpio_setpin(JSDEV_LED_CPU_GPIO_PIN, LED_OFF);
			break;

		case led_idle_end:
			(void)ns9xxx_gpio_setpin(JSDEV_LED_CPU_GPIO_PIN, LED_ON);
			break;
#endif

		case led_halted:
			break;

		default:
			break;
	}

	local_irq_restore(flags);
}
