/* linux/arch/arm/mach-s3c2410/a9m24x0_nand.c
 *
 * Copyright (C) 2005 by FS Forth-Systeme GmbH
 * All rights reserved.
 *
 * $Id: a9m24x0_nand.c,v 1.2 2005-09-21 14:22:53 jjaeger Exp $
 * @Author: Joachim Jaeger
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * @History:
 * derived from linux/arch/arm/mach-s3c2410/bast-nand.c, written by
 * Ben Dooks <ben@simtec.co.uk>
 * Modifications:
 *	23-Sep-2004 BJD  Initial support for NAND devices on BAST
 *	25-Sep-2004 BJD  Fixes for new mtd code
 *	27-Jan-2005 JJ	 Adapted to A9M24x0 modules
 *	22-Mar-2005 JJ	 New partitions
 *	21-Sep-2005 JJ	 New partitions
*/

//#define DEBUG

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/device.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/nand_ecc.h>
#include <linux/mtd/partitions.h>

#include <asm/arch/nand.h>

/* NAND Flash */

static int chip32MB_map[] = { 0 };
static int chip64MB_map[] = { 1 };
static int chip128MB_map[] = { 2 };

struct mtd_partition a9m24x0_default_nand_part[] = {
	[0] = {
		.name	= "u-boot",
		.size	= 256*1024,
		.offset	= 0,
		.mask_flags	= MTD_WRITEABLE,  /* force read-only */
	},
	[1] = {
		.name	= "kernel",
		.size	= (3*1024*1024)-(256*1024),
		.offset	= 256*1024,
		.mask_flags	= MTD_WRITEABLE,  /* force read-only */
	},
	[2] = {
		.name	= "rootfs",
		.offset	= 3*1024*1024,
		.size	= MTDPART_SIZ_FULL,
	}
};

static struct s3c2410_nand_set a9m24x0_nand_sets[] = {
	[0] = {
		.name		= "32MB_NAND",
		.nr_chips	= 1,
		.nr_map		= chip32MB_map,
		.nr_partitions	= ARRAY_SIZE(a9m24x0_default_nand_part),
		.partitions	= a9m24x0_default_nand_part
	},
	[1] = {
		.name		= "64MB_NAND",
		.nr_chips	= 1,
		.nr_map		= chip64MB_map,
		.nr_partitions	= ARRAY_SIZE(a9m24x0_default_nand_part),
		.partitions	= a9m24x0_default_nand_part
	},
	[2] = {
		.name		= "128MB_NAND",
		.nr_chips	= 1,
		.nr_map		= chip128MB_map,
		.nr_partitions	= ARRAY_SIZE(a9m24x0_default_nand_part),
		.partitions	= a9m24x0_default_nand_part
	}
};

static void a9m24x0_nand_select(struct s3c2410_nand_set *set, int chip)
{

	chip = set->nr_map[chip] & 3;

	pr_debug("a9m24x0_nand: selecting chip %d[%s] (set %p,%p)\n",
		 chip, set->name, set, set->nr_map);

}

static struct s3c2410_platform_nand a9m24x0_nand_info = {
	.tacls		= 30,
	.twrph0		= 40,
	.twrph1		= 20,
	.nr_sets	= ARRAY_SIZE(a9m24x0_nand_sets),
	.sets		= a9m24x0_nand_sets,
	.select_chip	= a9m24x0_nand_select,
};

void __init a9m24x0_init_nand(struct device *dev)
{
	printk(KERN_INFO "A9M24x0 NAND flash support\n");
	dev->platform_data = &a9m24x0_nand_info;
}

