/***********************************************************************
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 *
 * $Id: modarm9.c,v 1.14 2006-11-14 15:34:24 ludwigm Exp $
 * @Author: Markus Pietrek
 * @Descr: NOR Flash memory mapping for the ModARM9 family. Currently supported
 *         is the A9M8695
 *         Derived from sa1100-flash.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *  
 ***********************************************************************
 * @History:
 *    05/04/2005: Added UNC90 support by Pedro Perez de Heredia.
 *    09/08/2005: PPH : Changed header file for UNC90 machine.
 *    12/09/2005: Added CCXP270 support (Bernd Westermann)
 *    16/09/2005: CCXP270 mappings modified (Bernd Westermann)
 *    19/09/2005: CC9C support (Jonas Dietsche)
 *    25/09/2005: CCXP270 mappings modified (Bernd Westermann)
 *    08/08/2006: CCW9C support (Bernd Westermann)
 ***********************************************************************/

#include <linux/autoconf.h>

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#include <linux/ioport.h>	/* request_mem_region */

#define DRIVER_NAME	"modarm9"

#ifdef CONFIG_MACH_UNC90

#include <asm/arch/hardware.h>
#define FLASH_MEM_BASE_P	AT91_FLASH_BASE
#define FLASH_SIZE		0x001000000
#define	FLASH_BANK_WIDTH	2

static struct mtd_partition modarm9_partitions[] = {
	{
		.name       = "all",
		.offset     = 0x00000000,
	},
	{
		.name       = "u-boot",
		.offset     = 0x00000000,
		.size       = 0x00040000,	// 4 * 64kB for u-boot and boot.bin 
	},
	{
		.name       = "kernel",		// default kernel image (3MB)
		.offset     = 0x00040000,
		.size       = 0x00300000,
	},
	{
		.name       = "rootfs",		// rootfs (3MB)
		.offset     = 0x00340000,
		.size       = 0x00300000,
	},
 	{
		.name       = "flash_rw",	// jffs2 read/write partition (2MB)
		.offset     = 0x00640000,
		.size       = 0x00200000,
	},
	{
		.name       = "flash_ro",	// cramfs/romfs parttion (2MB)
		.offset     = 0x00840000,
		.size       = 0x00200000,
	},
	{
		.name       = "flash2_rw",	// jffs2 read/write partition (2MB)
		.offset     = 0x00a40000,
		.size       = 0x00200000,		
	},
	{
		.name       = "unused",
		.offset     = 0x00c40000,
	}
};
#endif /* CONFIG_MACH_UNC90 */

#ifdef CONFIG_ARCH_KS8695

#define FLASH_MEM_BASE_P	0x02800000
#define FLASH_SIZE		0x00800000
#define FLASH_BANK_WIDTH	2

static struct mtd_partition modarm9_partitions[] = {
	{
		.name       = "all",
		.offset     = 0x00000000,
	},
	{
		.name       = "linuxloader",
		.offset     = 0x00000000,
		.size       = 0x00010000, /* 64kB */
	},
	{
		.name       = "kernel",	/* default kernel image */
		.offset     = 0x00010000,
		.size       = 0x00100000,
	},
	{
		.name       = "initrd",
		.offset     = 0x00110000,
		.size       = 0x00100000,
	},
 	{
		.name       = "unused",
		.offset     = 0x00210000,
	}
};
#endif /* CONFIG_ARCH_KS8695P */

#ifdef CONFIG_MACH_CCXP

#define FLASH_MEM_BASE_P	0x00000000
#define FLASH_SIZE		0x02000000
#define	FLASH_BANK_WIDTH	4

static struct mtd_partition modarm9_partitions[] = {
	{
		.name       = "u-boot",
		.offset     = 0x00000000,
		.size       = 0x00040000, 
	},
	{
		.name       = "kernel",	
		.offset     = 0x00080000,
		.size       = 0x00280000,
	},
	{
		.name       = "rootfs",
		.offset     = 0x00300000,
		.size       = 0x1D00000,
	},
	{
		.name       = "eeprom",
		.offset     = 0x00040000,
		.size       = 0x00040000,
	},
	{
		.name       = "all",
		.offset     = 0x00000000,
		.size       = 0x2000000,
	}
};
#endif /* CONFIG_MACH_CCXP */

#ifdef CONFIG_MACH_CC9P9360DEV

#define FLASH_MEM_BASE_P	0x50000000
#define FLASH_SIZE		0x00400000
#define FLASH_BANK_WIDTH	2

static struct mtd_partition modarm9_partitions[] = {
	{
		.name       = "uboot",
		.offset     = 0x00000000,
		.size       = 0x00050000, /* 256kB u-boot + 64kB environment variables */
	},
	{
		.name       = "kernel",	/* default kernel image */
		.offset     = 0x00050000,
		.size       = 0x001B0000,
	},
	{
		.name       = "rootfs",
		.offset     = 0x00200000,
	}
};
#endif /* CONFIG_MACH_CC9P9360DEV */

#ifdef CONFIG_MACH_CC9C

#define FLASH_MEM_BASE_P	0x50000000
#define FLASH_SIZE		0x00400000
#define FLASH_BANK_WIDTH	2

static struct mtd_partition modarm9_partitions[] = {
	{
		.name       = "uboot",
		.offset     = 0x00000000,
		.size       = 0x00050000, /* 256kB u-boot + 64kB environment variables */
	},
	{
		.name       = "kernel",	/* default kernel image */
		.offset     = 0x00050000,
		.size       = 0x001B0000,
	},
	{
		.name       = "rootfs",
		.offset     = 0x00200000,
	}
};
#endif /* CONFIG_MACH_CC9C */

#ifdef CONFIG_MACH_CCW9C

#define FLASH_MEM_BASE_P	0x50000000
#define FLASH_SIZE		0x00400000
#define FLASH_BANK_WIDTH	2

static struct mtd_partition modarm9_partitions[] = {
	{
		.name       = "uboot",
		.offset     = 0x00000000,
		.size       = 0x00100000, /* 256kB u-boot + 64kB environment variables */
	},
	{
		.name       = "kernel",	/* default kernel image */
		.offset     = 0x00100000,
		.size       = 0x001B0000,
	},
	{
		.name       = "rootfs",
		.offset     = 0x002B0000,
	}
};
#endif /* CONFIG_MACH_CCW9C */

static struct map_info modarm9_map = 
{
	.name      = "modarm9",
	.phys 	   = FLASH_MEM_BASE_P, 
	.size 	   = FLASH_SIZE, 
	.bankwidth = FLASH_BANK_WIDTH,
};

static struct mtd_partition* parsed_parts;
static const char* part_probes[] = { "cmdlinepart", NULL };
static struct mtd_info* mtd_info;
static struct resource* mtd_res;

static int __init modarm9_mtd_init( void )
{
	const char* part_type = NULL;
	int nr_parts = 0;
	int ret = 0;

	printk( KERN_INFO DRIVER_NAME
		": Using NOR Flash device: %lu kB @ 0x%x\n",
		modarm9_map.size / 1024, (unsigned int)modarm9_map.phys );

	mtd_res = request_mem_region( modarm9_map.phys, modarm9_map.size,
				      modarm9_map.name );
	if( mtd_res == NULL ) {
		ret = -EBUSY;
		goto error;
	}

	modarm9_map.virt = (void __iomem *) ioremap( modarm9_map.phys,
						     modarm9_map.size );
	if( !modarm9_map.virt ) {
		ret = -ENOMEM;
		goto error_map;
	}
	
	simple_map_init( &modarm9_map );

	/* probe if there is really flash */

#ifdef CONFIG_MACH_CCXP
	mtd_info = NULL;
#else
	mtd_info = do_map_probe( "amd_flash", &modarm9_map );
#endif
	if( mtd_info == NULL ) {
		mtd_info = do_map_probe( "cfi_probe", &modarm9_map );
		if( mtd_info == NULL ) {
			ret = -ENXIO;
			goto error_probe;
		}
	}
	
	mtd_info->owner = THIS_MODULE;

	/* setup partitions */
#ifdef CONFIG_MTD_PARTITIONS
	nr_parts = parse_mtd_partitions( mtd_info, part_probes,
					 &parsed_parts, 0 );
	if( nr_parts > 0 )
		part_type = "dynamic";
	else {
		parsed_parts = modarm9_partitions;
		nr_parts = ARRAY_SIZE( modarm9_partitions );
		part_type = "static";
	}
#endif

	if( nr_parts == 0 ) {
		printk( KERN_INFO DRIVER_NAME ": no partition info "
			"available, registering whole flash\n" );
		add_mtd_device( mtd_info );
	} else {
		printk( KERN_INFO DRIVER_NAME ": using %s partition "
			"definition\n", part_type );
		add_mtd_partitions( mtd_info, parsed_parts, nr_parts );
	}

	return 0;

error_probe:
	iounmap( modarm9_map.virt );

error_map:
	release_resource( mtd_res );

error:
	return ret;
}

static void __exit modarm9_mtd_cleanup( void )
{

	del_mtd_partitions( mtd_info );

	if( parsed_parts )
		kfree( parsed_parts );

	iounmap( modarm9_map.virt );

	release_resource( mtd_res );

	parsed_parts = NULL;
}

module_init( modarm9_mtd_init );
module_exit( modarm9_mtd_cleanup );

MODULE_AUTHOR( "Markus Pietrek <mpietrek@fsforth.de>" );
MODULE_DESCRIPTION( "ModARM9 NOR flash mapping driver" );
MODULE_LICENSE( "GPL" );
