#include <asm/arch/irqs.h>
# include <asm/arch/ns9750_sys.h>
# include <asm/arch/ns9750_bbus.h>
# include <asm/arch/ns9750_bbus_bridge.h>
#ifdef CONFIG_ARCH_NS9750
# include <asm/arch/ns9750_usb.h>
#elif CONFIG_ARCH_NS9360
# include <asm/arch/ns9360_usb.h>
#endif

  .macro disable_fiq
  .endm

  .macro get_irqnr_and_base, irqnr, irqstat, base, tmp
  ldr \base, =(NS_SYS_INT_STAT_ACTIVE)
  ldr \irqstat, [\base]
  cmp \irqstat, #0 @ check if at least one interrupt is active
  beq 1001f

  @@ 1 interrupt is active, return it's number
  ldr \base, =(NS_SYS_ISRADDR)
  ldr \irqnr, [\base]
  cmp \irqnr, #IRQ_BBUS_AGGREGATE
  bne 1001f @ no BBUS interrupt, return AHB IRQ directly

  @@ now check which of the BBUS interrupt is the cause of our job
  ldr \base, =(NS_BBUS_BRIDGE_ISTAT)
  ldr \irqstat, [\base]
  cmp \irqstat, #0
  beq 1001f @ no interrupt known, must be spurious. Ignore it
  @@ @TODO. Must we clear spurious interrupts?

  tst \irqstat, #1 @ test for BBUS DMA Interrupt
  bne 1003f
  tst \irqstat, #2 @ test for USB Interrupt
  bne 1005f
  mov \tmp, #4 @ tmp will be bitmask
  mov \irqnr, #(IRQ_BBUS_S1RXE) @ we skipped USB and DMA

  @@ test BBUS interrupts
1002:
         @@ at least one bit must be 1, loop will terminate
  tst \irqstat, \tmp
  bne 1001f @ irqstat and status are set for irq received
  mov \tmp, \tmp, lsl #1
  add \irqnr, \irqnr, #1
  b 1002b

  @@ test BBUS DMA interrupts
1003:
  mov \tmp, #1 @ tmp will be bitmask
  mov \irqnr, #(IRQ_BBUS_DMA_1)
1004:
  @@ at least one bit must be 1, loop will terminate
  tst \irqstat, \tmp
  bne 1001f @ irqstat and status are set for irq received
  mov \tmp, \tmp, lsl #1
  add \irqnr, \irqnr, #1
  b 1004b

  @@ test USB interrupts
1005:
  @@ check which USB Interrupt is active (DMA Controller 2, FIFO)
  ldr \base, =(NS_USB_ISTAT)
  ldr \irqstat, [\base]
  mov \irqnr, #IRQ_USB_DMA
  tst \irqstat, #NS_USB_INT_GBL_DMA
  bne 1001f
  mov \irqnr, #IRQ_USB_HOST
  tst \irqstat, #NS_USB_INT_OHCI_IRQ
  bne 1001f
         @@ if it is not a DMA Interrupt, it must be FIFO
  mov \irqnr, #IRQ_USB_FIFO
1001:
  .endm


  .macro irq_prio_table
  .endm
