/*
 * include/asm-arm/arch-ns9xxx/regs-eth.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_REGSETH_H
#define __ASM_ARCH_REGSETH_H

#include <asm/arch/hardware.h>

/* Ethernet Communication Module */

/* Ethernet General Control Register #1 */
#define ETH_EGCR1	__REG(0xa0600000)

/* Ethernet General Control Register #1: Enable RX packet processing */
#define ETH_EGCR1_ERX		__REGBIT(31)
#define ETH_EGCR1_ERX_RESET		__REGVAL(ETH_EGCR1_ERX, 0)
#define ETH_EGCR1_ERX_EN		__REGVAL(ETH_EGCR1_ERX, 1)

/* Ethernet General Control Register #1: Enable receive DMA */
#define ETH_EGCR1_ERXDMA	__REGBIT(30)
#define ETH_EGCR1_ERXDMA_DIS		__REGVAL(ETH_EGCR1_ERXDMA, 0)
#define ETH_EGCR1_ERXDMA_EN		__REGVAL(ETH_EGCR1_ERXDMA, 1)

/* Ethernet General Control Register #1: Accept short (<64) receive frames */
#define ETH_EGCR1_ERXSHT	__REGBIT(28)
#define ETH_EGCR1_ERXSHT_DEN	__REGVAL(ETH_EGCR1_ERXSHT, 0)
#define ETH_EGCR1_ERXSHT_ACC	__REGVAL(ETH_EGCR1_ERXSHT, 1)

/* Ethernet General Control Register #1: Enable TX packet processing */
#define ETH_EGCR1_ETX		__REGBIT(23)
#define ETH_EGCR1_ETX_RESET		__REGVAL(ETH_EGCR1_ETX, 0)
#define ETH_EGCR1_ETX_EN		__REGVAL(ETH_EGCR1_ETX, 1)

/* Ethernet General Control Register #1: Enable transmit DMA */
#define ETH_EGCR1_ETXDMA	__REGBIT(22)
#define ETH_EGCR1_ETXDMA_DIS		__REGVAL(ETH_EGCR1_ETXDMA, 0)
#define ETH_EGCR1_ETXDMA_EN		__REGVAL(ETH_EGCR1_ETXDMA, 1)

/* Ethernet General Control Register #1: Always write as 1 */
#define ETH_EGCR1_ETXWM		__REGBIT(21)
#define ETH_EGCR1_ETXWM_DIS		__REGVAL(ETH_EGCR1_ETXWM, 0)
#define ETH_EGCR1_ETXWM_EN		__REGVAL(ETH_EGCR1_ETXWM, 1)

/* Ethernet General Control Register #1: Enable initialization of RX buffer descriptors */
#define ETH_EGCR1_ERXINIT	__REGBIT(19)
#define ETH_EGCR1_ERXINIT_INIT		__REGVAL(ETH_EGCR1_ERXINIT, 0)

/* Ethernet General Control Register #1: Ethernet interface mode */
#define ETH_EGCR1_PHYMODE	__REGBITS(15, 14)
#define ETH_EGCR1_PHYMODE_10100MII	__REGVAL(ETH_EGCR1_PHYMODE, 0)
#define ETH_EGCR1_PHYMODE_10100RMII	__REGVAL(ETH_EGCR1_PHYMODE, 1)

/* Ethernet General Control Register #1: Align RX data */
#define ETH_EGCR1_RXALIGN	__REGBIT(10)
#define ETH_EGCR1_RXALIGN_STD		__REGVAL(ETH_EGCR1_RXALIGN, 0)
#define ETH_EGCR1_RXALIGN_PAD		__REGVAL(ETH_EGCR1_RXALIGN, 1)

/* Ethernet General Control Register #1: MAC host interface soft reset */
#define ETH_EGCR1_MACHRST	__REGBIT(9)
#define ETH_EGCR1_MACHRST_RESTR		__REGVAL(ETH_EGCR1_MACHRST, 0)
#define ETH_EGCR1_MACHRST_RESET		__REGVAL(ETH_EGCR1_MACHRST, 1)

/* Ethernet General Control Register #1: Insert transmit source address */
#define ETH_EGCR1_ITXA		__REGBIT(8)
#define ETH_EGCR1_ITXA_FIFO		__REGVAL(ETH_EGCR1_ITXA, 0)
#define ETH_EGCR1_ITXA_STA		__REGVAL(ETH_EGCR1_ITXA, 1)

/* Ethernet General Control Register #2 */
#define ETH_EGCR2	__REG(0xa0600004)
/* Ethernet General Control Register #2: Clear transmit error */
#define ETH_EGCR2_TCLER		__REGBIT(3)

/* Ethernet General Control Register #2: Enable statistics counter clear on read */
#define ETH_EGCR2_AUTOZ		__REGBIT(2)

/* Ethernet General Control Register #2: Clear statistics counters */
#define ETH_EGCR2_CLRCNT	__REGBIT(1)

/* Ethernet General Control Register #2: Enable statistics counters */
#define ETH_EGCR2_STEN		__REGBIT(0)

/* Ethernet General Status register */
#define ETH_EGSR	__REG(0xa0600008)
#define ETH_EGSR_RXINIT		__REGBIT(20)

/* Ethernet General Status register: */
#define ETH_EGSR_RXINIT		__REGBIT(20)

/* Ethernet Transmit Status register */
#define ETH_ETSR	__REG(0xa0600018)

/* Ethernet Transmit Status register: Frame transmitted OK */
#define ETH_ETSR_TXOK		__REGBIT(15)

/* Ethernet Transmit Status register: Broadcast frame transmitted */
#define ETH_ETSR_TXBR		__REGBIT(14)

/* Ethernet Transmit Status register: Multicast frame transmitted */
#define ETH_ETSR_TXMC		__REGBIT(13)

/* Ethernet Transmit Status register: TX abort -- late collision */
#define ETH_ETSR_TXAL		__REGBIT(12)

/* Ethernet Transmit Status register: TX abort -- excessive deferral */
#define ETH_ETSR_TXAED		__REGBIT(11)

/* Ethernet Transmit Status register: TX abort -- excessive collisions */
#define ETH_ETSR_TXAEC		__REGBIT(10)

/* Ethernet Transmit Status register: TX abort -- underrun */
#define ETH_ETSR_TXAUR		__REGBIT(9)

/* Ethernet Transmit Status register: TX abort -- jumbo */
#define ETH_ETSR_TXAJ		__REGBIT(8)

/* Ethernet Transmit Status register: Transmit frame deferred */
#define ETH_ETSR_TXDEF		__REGBIT(6)

/* Ethernet Transmit Status register: Transmit CRC error */
#define ETH_ETSR_TXCRC		__REGBIT(5)

/* Ethernet Transmit Status register: Transmit collision count */
#define ETH_ETSR_TXCOLC		__REGBITS(3, 0)

/* Ethernet Receive Status register */
#define ETH_ERSR	__REG(0xa060001c)

/* MAC Configuration Register #1 */
#define ETH_MAC1	__REG(0xa0600400)

/* MAC Configuration Register #1: Soft reset */
#define ETH_MAC1_SRST		__REGBIT(15)

/* MAC Configuration Register #1: Receive enable */
#define ETH_MAC1_RXEN		__REGBIT(0)
#define ETH_MAC1_RXEN_EN		__REGVAL(ETH_MAC1_RXEN, 1)
#define ETH_MAC1_RXEN_DIS		__REGVAL(ETH_MAC1_RXEN, 0)

/* MAC Configuration Register #2 */
#define ETH_MAC2	__REG(0xa0600404)
#define ETH_MAC2_PADEN		__REGBIT(5)
#define ETH_MAC2_CRCEN		__REGBIT(4)
#define ETH_MAC2_FULLD		__REGBIT(0)

/* Back-to-Back Inter-Packet-Gap register */
#define ETH_IPGT	__REG(0xa0600408)
#define ETH_IPGT_MASK		__REGBITS(6, 0)

/* Non-Back-to-Back Inter-Packet-Gap register */
#define ETH_IPGR	__REG(0xa060040c)

/* Collision Window/Retry register */
#define ETH_CLRT	__REG(0xa0600410)

/* Maximum Frame register */
#define ETH_MAXF	__REG(0xa0600414)

/* PHY Support register */
#define ETH_SUPP	__REG(0xa0600418)

/* MII Management Configuration register */
#define ETH_MCFG	__REG(0xa0600420)

/* MII Management Configuration register: Reset MII management block */
#define ETH_MCFG_RMIIM		__REGBIT(15)

/* MII Management Configuration register: Clock select */
#define ETH_MCFG_CLKS		__REGBITS(4, 2)
#define ETH_MCFG_CLKS_DIV4		__REGVAL(ETH_MCFG_CLKS, 0)
#define ETH_MCFG_CLKS_DIV6		__REGVAL(ETH_MCFG_CLKS, 2)
#define ETH_MCFG_CLKS_DIV8		__REGVAL(ETH_MCFG_CLKS, 3)
#define ETH_MCFG_CLKS_DIV10		__REGVAL(ETH_MCFG_CLKS, 4)
#define ETH_MCFG_CLKS_DIV20		__REGVAL(ETH_MCFG_CLKS, 5)
#define ETH_MCFG_CLKS_DIV30		__REGVAL(ETH_MCFG_CLKS, 6)
#define ETH_MCFG_CLKS_DIV40		__REGVAL(ETH_MCFG_CLKS, 7)

/* MII Management Command register */
#define ETH_MCMD	__REG(0xa0600424)
/* MII Management Command register: Automatically scan for read data */
#define ETH_MCMD_SCAN		__REGBIT(1)
/* MII Management Command register: Single scan for read data */
#define ETH_MCMD_READ		__REGBIT(0)

/* MII Management Address register */
#define ETH_MADR	__REG(0xa0600428)

/* MII Management Write Data register */
#define ETH_MWTD	__REG(0xa060042c)

/* MII Management Read Data register */
#define ETH_MRDD	__REG(0xa0600430)

/* MII Management Indicators register */
#define ETH_MIND	__REG(0xa0600434)
#define ETH_MIND_BUSY	__REGBIT(0)

/* Station Address Register #1 */
#define ETH_SA1		__REG(0xa0600440)

/* Station Address Register #2 */
#define ETH_SA2		__REG(0xa0600444)

/* Station Address register #3 */
#define ETH_SA3		__REG(0xa0600448)

/* Station Address Filter register */
#define ETH_SAFR	__REG(0xa0600500)
/* Station Address Filter register: Enable promiscuous mode; receive all frames */
#define ETH_SAFR_PRO		__REGBIT(3)
#define ETH_SAFR_PRO_YES		__REGVAL(ETH_SAFR_PRO, 1)
#define ETH_SAFR_PRO_NO			__REGVAL(ETH_SAFR_PRO, 0)

/* Station Address Filter register: Accept all multicast frames */
#define ETH_SAFR_PRM		__REGBIT(2)
#define ETH_SAFR_PRM_YES		__REGVAL(ETH_SAFR_PRM, 1)
#define ETH_SAFR_PRM_NO			__REGVAL(ETH_SAFR_PRM, 0)

/* Station Address Filter register: Accept multicast frames using the hash table */
#define ETH_SAFR_PRA		__REGBIT(1)
#define ETH_SAFR_PRA_YES		__REGVAL(ETH_SAFR_PRA, 1)
#define ETH_SAFR_PRA_NO			__REGVAL(ETH_SAFR_PRA, 0)

/* Station Address Filter register: Accept all broadcast frames */
#define ETH_SAFR_BROAD		__REGBIT(0)
#define ETH_SAFR_BROAD_YES		__REGVAL(ETH_SAFR_BROAD, 1)
#define ETH_SAFR_BROAD_NO		__REGVAL(ETH_SAFR_BROAD, 0)

/* Hash Table Register #1 */
#define ETH_HT1		__REG(0xa0600504)

/* Hash Table Register #2 */
#define ETH_HT2		__REG(0xa0600508)

/* Statistics Register Transmit & receive */
#define ETH_STATTR64	__REG(0xa0600680)
#define ETH_STATTR127	__REG(0xa0600684)
#define ETH_STATTR255	__REG(0xa0600688)
#define ETH_STATTR511	__REG(0xa060068c)
#define ETH_STATTR1K	__REG(0xa0600690)
#define ETH_STATTRMAX	__REG(0xa0600694)
#define ETH_STATTRMGV	__REG(0xa0600698)
#define ETH_STATRBYT	__REG(0xa060069c)
#define ETH_STATRPKT	__REG(0xa06006a0)
#define ETH_STATRFCS	__REG(0xa06006a4)
#define ETH_STATRMCA	__REG(0xa06006a8)
#define ETH_STATRBCA	__REG(0xa06006ac)
#define ETH_STATRXCF	__REG(0xa06006b0)
#define ETH_STATRXPF	__REG(0xa06006b4)
#define ETH_STATRXUO	__REG(0xa06006b8)
#define ETH_STATRALN	__REG(0xa06006bc)
#define ETH_STATRFLR	__REG(0xa06006c0)
#define ETH_STATRCDE	__REG(0xa06006c4)
#define ETH_STATRCSE	__REG(0xa06006c8)
#define ETH_STATRUND	__REG(0xa06006cc)
#define ETH_STATROVR	__REG(0xa06006d0)
#define ETH_STATRFRG	__REG(0xa06006d4)
#define ETH_STATRJBR	__REG(0xa06006d8)
#define ETH_STATTBYT	__REG(0xa06006e0)
#define ETH_STATTPKT	__REG(0xa06006e4)
#define ETH_STATTMCA	__REG(0xa06006e8)
#define ETH_STATTBCA	__REG(0xa06006ec)
#define ETH_STATTDFR	__REG(0xa06006f4)
#define ETH_STATTEDF	__REG(0xa06006f8)
#define ETH_STATTSCL	__REG(0xa06006fc)
#define ETH_STATTMCL	__REG(0xa0600700)
#define ETH_STATTLCL	__REG(0xa0600704)
#define ETH_STATTXCL	__REG(0xa0600708)
#define ETH_STATTNCL	__REG(0xa060070c)
#define ETH_STATTJBR	__REG(0xa0600718)
#define ETH_STATTFCS	__REG(0xa060071c)
#define ETH_STATTOVR	__REG(0xa0600724)
#define ETH_STATTUND	__REG(0xa0600728)
#define ETH_STATTFRG	__REG(0xa060072c)
#define ETH_STATCAR1	__REG(0xa0600730)
#define ETH_STATCAR2	__REG(0xa0600734)
#define ETH_STATCAM1	__REG(0xa0600738)
#define ETH_STATCAM2	__REG(0xa060073c)

/* RX_A Buffer Descriptor Pointer register */
#define ETH_RXAPTR	__REG(0xa0600a00)

/* RX_B Buffer Descriptor Pointer register */
#define ETH_RXBPTR	__REG(0xa0600a04)

/* RX_C Buffer Descriptor Pointer register */
#define ETH_RXCPTR	__REG(0xa0600a08)

/* RX_D Buffer Descriptor Pointer register */
#define ETH_RXDPTR	__REG(0xa0600a0c)

/* Ethernet Interrupt Status register */
#define ETH_EINTR	__REG(0xa0600a10)
/* Ethernet Interrupt Status register: RX data FIFO overflowed */
#define ETH_EINTR_RXOVLDATA	__REGBIT(25)
/* Ethernet Interrupt Status register: RX status FIFO overflowed. */
#define ETH_EINTR_RXOVLSTAT	__REGBIT(24)
/* Ethernet Interrupt Status register: I bit set in Receive Buffer Descriptor and buffer closed. */
#define ETH_EINTR_RXBUFC	__REGBIT(23)
/* Ethernet Interrupt Status register: Complete receive frame stored in pool A */
#define ETH_EINTR_RXDONEA	__REGBIT(22)
/* Ethernet Interrupt Status register: Complete receive frame stored in pool B */
#define ETH_EINTR_RXDONEB	__REGBIT(21)
/* Ethernet Interrupt Status register: Complete receive frame stored in pool C */
#define ETH_EINTR_RXDONEC	__REGBIT(20)
/* Ethernet Interrupt Status register: Complete receive frame stored in pool D */
#define ETH_EINTR_RXDONED	__REGBIT(19)
/* Ethernet Interrupt Status register: No buffer is available */
#define ETH_EINTR_RXNOBUF	__REGBIT(18)
/* Ethernet Interrupt Status register: All four buffer rings are disabled or full. */
#define ETH_EINTR_RXBUFFUL	__REGBIT(17)
/* Ethernet Interrupt Status register: New frame available in the RX_FIFO. */
#define ETH_EINTR_RXBR		__REGBIT(16)
/* Ethernet Interrupt Status register: Statistics counter overflow. */
#define ETH_EINTR_STOVFL	__REGBIT(6)
/* Ethernet Interrupt Status register: I bit set in Transmit Buffer Descriptor and buffer closed. */
#define ETH_EINTR_TXBUFC	__REGBIT(4)
/* Ethernet Interrupt Status register: F bit not set in the Transmit Buffer Descriptor */
#define ETH_EINTR_TXBUFNR	__REGBIT(3)
/* Ethernet Interrupt Status register: Frame transmission complete. */
#define ETH_EINTR_TXDONE	__REGBIT(2)
/* Ethernet Interrupt Status register: Last frame not transmitted successfully. */
#define ETH_EINTR_TXERR		__REGBIT(1)
/* Ethernet Interrupt Status register: TX_WR logic has no frame to transmit. */
#define ETH_EINTR_TXIDLE	__REGBIT(0)

/* Ethernet Interrupt Enable register */
#define ETH_EINTREN	__REG(0xa0600a14)
/* Ethernet Interrupt Enable register: Enable the RXOVFL_DATA interrupt bit. */
#define ETH_EINTREN_RXOVFLDATA		__REGBIT(25)
/* Ethernet Interrupt Enable register: Enable the RXOVFL_STATUS interrupt bit. */
#define ETH_EINTREN_RXOVFLSTAT		__REGBIT(24)
/* Ethernet Interrupt Enable register: Enable the RXBUFC interrupt bit. */
#define ETH_EINTREN_RXBUFC		__REGBIT(23)
/* Ethernet Interrupt Enable register: Enable the RXDONEA interrupt bit. */
#define ETH_EINTREN_RXDONEA		__REGBIT(22)
/* Ethernet Interrupt Enable register: Enable the RXDONEB interrupt bit. */
#define ETH_EINTREN_RXDONEB		__REGBIT(21)
/* Ethernet Interrupt Enable register: Enable the RXDONEC interrupt bit. */
#define ETH_EINTREN_RXDONEC		__REGBIT(20)
/* Ethernet Interrupt Enable register: Enable the RXDONED interrupt bit. */
#define ETH_EINTREN_RXDONED		__REGBIT(19)
/* Ethernet Interrupt Enable register: Enable the RXNOBUF interrupt bit. */
#define ETH_EINTREN_RXNOBUF		__REGBIT(18)
/* Ethernet Interrupt Enable register: Enable the RXBUFFUL interrupt bit. */
#define ETH_EINTREN_RXBUFFUL		__REGBIT(17)
/* Ethernet Interrupt Enable register: Enable the RXBR interrupt bit. */
#define ETH_EINTREN_RXBR		__REGBIT(16)
/* Ethernet Interrupt Enable register: Enable the STOVFL interrupt bit. */
#define ETH_EINTREN_STOVFL		__REGBIT(6)
/* Ethernet Interrupt Enable register: Enable the TXBUFC interrupt bit. */
#define ETH_EINTREN_TXBUFC		__REGBIT(4)
/* Ethernet Interrupt Enable register: Enable the TXBUFNR interrupt bit. */
#define ETH_EINTREN_TXBUFNR		__REGBIT(3)
/* Ethernet Interrupt Enable register: Enable the TXDONE interrupt bit. */
#define ETH_EINTREN_TXDONE		__REGBIT(2)
/* Ethernet Interrupt Enable register: Enable the TXERR interrupt bit. */
#define ETH_EINTREN_TXERR		__REGBIT(1)
/* Ethernet Interrupt Enable register: Enable the TXIDLE interrupt bit. */
#define ETH_EINTREN_TXIDLE		__REGBIT(0)



/* TX Buffer Descriptor Pointer register */
#define ETH_TXPTR	__REG(0xa0600a18)

/* TX Recover Buffer Descriptor Pointer register */
#define ETH_TXRPTR	__REG(0xa0600a1c)

/* TX Error Buffer Descriptor Pointer register */
#define ETH_TXERBD	__REG(0xa0600a20)

/* TX Stall Buffer Descriptor Pointer register */
#define ETH_TXSPTR	__REG(0xa0600a24)

/* RX_A Buffer Descriptor Pointer Offset register */
#define ETH_RXAOFF	__REG(0xa0600a28)

/* RX_B Buffer Descriptor Pointer Offset register */
#define ETH_RXBOFF	__REG(0xa0600a2c)

/* RX_C Buffer Descriptor Pointer Offset register */
#define ETH_RXCOFF	__REG(0xa0600a30)

/* RX_D Buffer Descriptor Pointer Offset register */
#define ETH_RXDOFF	__REG(0xa0600a34)

/* Transmit Buffer Descriptor Pointer Offset register */
#define ETH_TXOFF	__REG(0xa0600a38)

/* RX Free Buffer register */
#define ETH_RXFREE	__REG(0xa0600a3c)

/* TX Buffer Descriptor RAM (256 locations) */
#define ETH_TXBD	__REG(0xa0601000)

#endif /* ifndef __ASM_ARCH_REGSETH_H */
