/*
 * linux/include/asm-arm/arch-ns9xxx/regs-i2c.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#ifndef __ASM_ARCH_REGSI2C_H
#define __ASM_ARCH_REGSI2C_H

#include <asm/arch/hardware.h>

/* I2C Module registers and addresses */
#define I2C_BASE_PA		(0x90500000)
#define I2C_BASE_VA		io_p2v(I2C_BASE_PA)
#define I2C_REG_LEN		(0x10)


/* Command Transmit Data Register */
#define I2C_TX			__REG(0x90500000)

/* Command Receive Data Register */
#define I2C_RX			__REG(0x90500000)

/* Master Address Register */
#define I2C_MSTRAD		__REG(0x90500004)

/* Slave Address Register */
#define I2C_SLAVEAD		__REG(0x90500008)

/* Configuration Register */
#define I2C_CFG			__REG(0x9050000c)


/* register bit fields */
/* XXX modify to use __REGBIT macro */

#define NS_I2C_DATA_TX_PIPE		(0x8000)
#define NS_I2C_DATA_TX_DLEN		(0x4000)
#define NS_I2C_DATA_TX_VAL		(0x2000)
#define NS_I2C_DATA_TX_CMD_MA		(0x1F00)
#define NS_I2C_DATA_TX_CMD_M_NOP	(0x0000)
#define NS_I2C_DATA_TX_CMD_M_READ	(0x0400)
#define NS_I2C_DATA_TX_CMD_M_WRITE	(0x0500)
#define NS_I2C_DATA_TX_CMD_M_STOP	(0x0600)
#define NS_I2C_DATA_TX_CMD_S_NOP	(0x1000)
#define NS_I2C_DATA_TX_CMD_S_STOP	(0x1600)
#define NS_I2C_DATA_TX_DATA_MA		(0x00FF)
#define NS_I2C_DATA_TX_CMD_SH		(0x08) /* shift value for CMD_MA */

#define NS_I2C_DATA_RX_BSTS		(0x8000)
#define NS_I2C_DATA_RX_RDE		(0x4000)
#define NS_I2C_DATA_RX_SCMDL		(0x2000)
#define NS_I2C_DATA_RX_MCMDL		(0x1000)
#define NS_I2C_DATA_RX_IRQCD_MA		(0x0F00)
#define NS_I2C_DATA_RX_IRQCD_NO_IRQ   	(0x0000) /* no single bit fields */
#define NS_I2C_DATA_RX_IRQCD_M_ARBIT	(0x0100)
#define NS_I2C_DATA_RX_IRQCD_M_NO_ACK	(0x0200)
#define NS_I2C_DATA_RX_IRQCD_M_TX_DATA	(0x0300)
#define NS_I2C_DATA_RX_IRQCD_M_RX_DATA	(0x0400)
#define NS_I2C_DATA_RX_IRQCD_M_CMD_ACK	(0x0500)
#define NS_I2C_DATA_RX_IRQCD_S_RX_ABORT	(0x0800)
#define NS_I2C_DATA_RX_IRQCD_S_CMD_REQ	(0x0900)
#define NS_I2C_DATA_RX_IRQCD_S_NO_ACK	(0x0A00)
#define NS_I2C_DATA_RX_IRQCD_S_TX_DATA1	(0x0B00)
#define NS_I2C_DATA_RX_IRQCD_S_RX_DATA1	(0x0C00)
#define NS_I2C_DATA_RX_IRQCD_S_TX_DATA	(0x0D00)
#define NS_I2C_DATA_RX_IRQCD_S_RX_DATA	(0x0E00)
#define NS_I2C_DATA_RX_IRQCD_S_CGA  	(0x0F00)
#define NS_I2C_DATA_RX_DATA_MA		(0x00FF)

#define NS_I2C_MASTER_MDA_MA		(0x07FE)
#define NS_I2C_MASTER_MAM		(0x0001)
#define NS_I2C_MASTER_MDA_SH		(0x01)

#define NS_I2C_SLAVE_GCA		(0x0800)
#define NS_I2C_SLAVE_SDA_MA		(0x07FE)
#define NS_I2C_SLAVE_SAM		(0x0001)
#define NS_I2C_SLAVE_SDA_SH		(0x01)

#define NS_I2C_CFG_IRQD			(0x8000)
#define NS_I2C_CFG_TMDE			(0x4000)
#define NS_I2C_CFG_VSCD			(0x2000)
#define NS_I2C_CFG_SFW_MA		(0x1E00)
#define NS_I2C_CFG_CLREF_MA		(0x01FF)
#define NS_I2C_CFG_SFW_SH		(0x09)

#endif /* ifndef __ASM_ARCH_REGSI2C_H */
