/*
 * include/asm-arm/arch-ns9xxx/regs-sys.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_REGSSYS_H
#define __ASM_ARCH_REGSSYS_H

#include <asm/arch/hardware.h>


/* System control module, register base addresses and offsets */
#define SYS_MODULE_BASE_PA	(0xa0900000)
#define SYS_MODULE_BASE_VA	io_p2v(SYS_MODULE_BASE_PA)

#define SYS_SWC_BASE_PA		(0xa0900174)
#define SYS_SWC_BASE_VA		io_p2v(SYS_SWC_BASE_PA)
#define SYS_SWC_REG_LEN		(0x00000008)


/* System Control Module */

/* AHB Arbiter Gen Configuration */
#define SYS_AHBAGENCONF		__REG(0xa0900000)

/* BRC */
#define SYS_BRC(x)		__REG2(0xa0900004, (x))

/* Timer x Reload Count register */
#define SYS_TRC(x)		__REG2(0xa0900044, (x))

/* Timer x Read register */
#define SYS_TR(x)		__REG2(0xa0900084, (x))

/* Interrupt Vector Address Register Level x */
#define SYS_IVA(x)		__REG2(0xa09000c4, (x))

/* Interrupt Configuration registers */
#define SYS_IC(x)		__REG2(0xa0900144, (x))

/* ISRADDR */
#define SYS_ISRADDR		__REG(0xa0900164)

/* Interrupt Status Active */
#define SYS_ISA			__REG(0xa0900168)

/* Interrupt Status Raw */
#define SYS_ISR			__REG(0xa090016c)

/* Timer Interrupt Status register */
#define SYS_TIS			__REG(0xa0900170)

/* Software Watchdog Configuration register */
#define SYS_SWC			__REG(0xa0900174)

#define SYS_SWC_EN		__REGBIT(7)
#define SYS_SWC_EN_ON		__REGVAL(SYS_SWC_EN, 1)
#define SYS_SWC_EN_OFF		__REGVAL(SYS_SWC_EN, 0)
#define SYS_SWC_IC		__REGBIT(5)
#define SYS_SWC_IC_ON		__REGVAL(SYS_SWC_IC, 1)
#define SYS_SWC_IC_OFF		__REGVAL(SYS_SWC_IC, 0)
#define SYS_SWC_IRESP		__REGBIT(4)
#define SYS_SWC_IRESP_RST	__REGVAL(SYS_SWC_IRESP, 1)
#define SYS_SWC_IRESP_IRQ	__REGVAL(SYS_SWC_IRESP, 0)
#define SYS_SWC_TCS		__REGBITS(2, 0)
#define SYS_SWC_CPUCLK_DIV_2 	_REGVAL(SYS_SWC_TCS, 0)
#define SYS_SWC_CPUCLK_DIV_4 	_REGVAL(SYS_SWC_TCS, 1)
#define SYS_SWC_CPUCLK_DIV_8 	_REGVAL(SYS_SWC_TCS, 2)
#define SYS_SWC_CPUCLK_DIV_16	_REGVAL(SYS_SWC_TCS, 3)
#define SYS_SWC_CPUCLK_DIV_32	_REGVAL(SYS_SWC_TCS, 4)
#define SYS_SWC_CPUCLK_DIV_64	_REGVAL(SYS_SWC_TCS, 5)

/* Software Watchdog Timer register */
#define SYS_SWT			__REG(0xa0900178)

/* Clock Configuration register */
#define SYS_CCR			__REG(0xa090017c)


#define SYS_CLK_RTC		__REGBIT(11)
#define SYS_CLK_RTC_ON		__REGVAL(SYS_CLK_RTC, 1)
#define SYS_CLK_RTC_OFF		__REGVAL(SYS_CLK_RTC, 0)
#define SYS_CLK_I2C		__REGBIT(10)
#define SYS_CLK_I2C_ON		__REGVAL(SYS_CLK_I2C, 1)
#define SYS_CLK_I2C_OFF		__REGVAL(SYS_CLK_I2C, 0)
#define SYS_CLK_LPCS		__REGBITS(9, 7)
#define SYS_CLK_LPCS_AHB	__REGVAL(SYS_CLK_LPCS, 0)
#define SYS_CLK_LPCS_AHB_DIV2	__REGVAL(SYS_CLK_LPCS, 1)
#define SYS_CLK_LPCS_AHB_DIV4	__REGVAL(SYS_CLK_LPCS, 2)
#define SYS_CLK_LPCS_AHB_DIV8	__REGVAL(SYS_CLK_LPCS, 3)
#define SYS_CLK_LPCS_EXT	__REGVAL(SYS_CLK_LPCS, 4)
#define SYS_CLK_BBC		__REGBIT(6)
#define SYS_CLK_BBC_ON		__REGVAL(SYS_CLK_BBC, 1)
#define SYS_CLK_BBC_OFF		__REGVAL(SYS_CLK_BBC, 0)
#define SYS_CLK_LCC		__REGBIT(5)
#define SYS_CLK_LCC_ON		__REGVAL(SYS_CLK_LCC, 1)
#define SYS_CLK_LCC_OFF		__REGVAL(SYS_CLK_LCC, 0)
#define SYS_CLK_MCC		__REGBIT(4)
#define SYS_CLK_MCC_ON		__REGVAL(SYS_CLK_MCC, 1)
#define SYS_CLK_MCC_OFF		__REGVAL(SYS_CLK_MCC, 0)
#define SYS_CLK_MACC		__REGBIT(0)
#define SYS_CLK_MACC_ON		__REGVAL(SYS_CLK_MACC, 1)
#define SYS_CLK_MACC_OFF	__REGVAL(SYS_CLK_MACC, 0)

/* Reset and sleep control register */
#define SYS_RSTSLEEP		__REG(0xa0900180)

#define SYS_RST_LCDC		__REGBIT(5)
#define SYS_RST_LCDC_ON		__REGVAL(SYS_RST_LCDC, 0)
#define SYS_RST_LCDC_OFF	__REGVAL(SYS_RST_LCDC, 1)

#define SYS_RST_MACM		__REGBIT(0)
#define SYS_RST_MACM_ON		__REGVAL(SYS_RST_MACM, 0)
#define SYS_RST_MACM_OFF	__REGVAL(SYS_RST_MACM, 1)

/* PLL Configuration register */
#define SYS_PLL			__REG(0xa0900188)

/* PLL FS status */
#define SYS_PLL_FS		__REGBITS(24, 23)

/* PLL ND status */
#define SYS_PLL_ND		__REGBITS(20, 16)

/* PLL Configuration register: PLL SW change */
#define SYS_PLL_SWC		__REGBIT(15)
#define SYS_PLL_SWC_NO		__REGVAL(SYS_PLL_SWC, 0)
#define SYS_PLL_SWC_YES		__REGVAL(SYS_PLL_SWC, 1)

/* Timer x Control register */
#define SYS_TC(x)		__REG2(0xa0900190, (x))

/* Timer x Control register: Timer enable */
#define SYS_TCx_TEN		__REGBIT(15)
// PPH XXX is the following definition correct??? revisit!
#define SYS_TCx_TEN_DIS		__REGVAL(SYS_TCx_TEN, 1)
#define SYS_TCx_TEN_EN		__REGVAL(SYS_TCx_TEN, 1)

/* Timer x Control register: CPU debug mode */
#define SYS_TCx_TDBG		__REGBIT(10)
#define SYS_TCx_TDBG_CONT	__REGVAL(SYS_TCx_TDBG, 0)
#define SYS_TCx_TDBG_STOP	__REGVAL(SYS_TCx_TDBG, 1)

/* Timer x Control register: Interrupt clear */
#define SYS_TCx_INTC		__REGBIT(9)
#define SYS_TCx_INTC_UNSET	__REGVAL(SYS_TCx_INTC, 0)
#define SYS_TCx_INTC_SET	__REGVAL(SYS_TCx_INTC, 1)

/* Timer x Control register: Timer clock select */
#define SYS_TCx_TLCS		__REGBITS(8, 6)
#define SYS_TCx_TLCS_CPU	__REGVAL(SYS_TCx_TLCS, 0) /* CPU clock */
#define SYS_TCx_TLCS_DIV2	__REGVAL(SYS_TCx_TLCS, 1) /* CPU clock / 2 */
#define SYS_TCx_TLCS_DIV4	__REGVAL(SYS_TCx_TLCS, 2) /* CPU clock / 4 */
#define SYS_TCx_TLCS_DIV8	__REGVAL(SYS_TCx_TLCS, 3) /* CPU clock / 8 */
#define SYS_TCx_TLCS_DIV16	__REGVAL(SYS_TCx_TLCS, 4) /* CPU clock / 16 */
#define SYS_TCx_TLCS_DIV32	__REGVAL(SYS_TCx_TLCS, 5) /* CPU clock / 32 */
#define SYS_TCx_TLCS_DIV64	__REGVAL(SYS_TCx_TLCS, 6) /* CPU clock / 64 */
#define SYS_TCx_TLCS_EXT	__REGVAL(SYS_TCx_TLCS, 7) /* CPU clock / 64 */

/* Timer x Control register: Timer mode */
#define SYS_TCx_TM		__REGBITS(5, 4)
#define SYS_TCx_TM_IEE		__REGVAL(SYS_TCx_TM, 0)		/* Internal timer or external event */
#define SYS_TCx_TM_ELL		__REGVAL(SYS_TCx_TM, 1)		/* External low-level, gated timer */
#define SYS_TCx_TM_EHL		__REGVAL(SYS_TCx_TM, 2)		/* External high-level, gated timer */
#define SYS_TCx_TM_CONCAT	__REGVAL(SYS_TCx_TM, 3)		/* Concatenate the lower timer.
								   Not applicable on timer 0. */
/* Timer x Control register: Interrupt select */
#define SYS_TCx_INTS		__REGBIT(3)
#define SYS_TCx_INTS_DIS	__REGVAL(SYS_TCx_INTS, 0)
#define SYS_TCx_INTS_EN		__REGVAL(SYS_TCx_INTS, 1)

/* Timer x Control register: Up/down select */
#define SYS_TCx_UDS		__REGBIT(2)
#define SYS_TCx_UDS_UP		__REGVAL(SYS_TCx_UDS, 0)
#define SYS_TCx_UDS_DOWN	__REGVAL(SYS_TCx_UDS, 1)

/* Timer x Control register: 32- or 16-bit timer */
#define SYS_TCx_TSZ		__REGBIT(1)
#define SYS_TCx_TSZ_16		__REGVAL(SYS_TCx_TSZ, 0)
#define SYS_TCx_TSZ_32		__REGVAL(SYS_TCx_TSZ, 1)

/* Timer x Control register: Reload enable */
#define SYS_TCx_REN		__REGBIT(0)
#define SYS_TCx_REN_DIS		__REGVAL(SYS_TCx_REN, 0)
#define SYS_TCx_REN_EN		__REGVAL(SYS_TCx_REN, 1)

/* System Memory Chip Select x Dynamic Memory Base */
#define SYS_SMCSDMB(x)		__REG2(0xa09001d0, (x) << 1)

/* System Memory Chip Select x Dynamic Memory Mask */
#define SYS_SMCSDMM(x)		__REG2(0xa09001d4, (x) << 1)

/* System Memory Chip Select x Static Memory Base */
#define SYS_SMCSSMB(x)		__REG2(0xa09001f0, (x) << 1)

/* System Memory Chip Select x Static Memory Base: Chip select x base */
#define SYS_SMCSSMB_CSxB	__REGBITS(31, 12)

/* System Memory Chip Select x Static Memory Mask */
#define SYS_SMCSSMM(x)		__REG2(0xa09001f4, (x) << 1)

/* System Memory Chip Select x Static Memory Mask: Chip select x mask */
#define SYS_SMCSSMM_CSxM	__REGBITS(31, 12)

/* System Memory Chip Select x Static Memory Mask: Chip select x enable */
#define SYS_SMCSSMM_CSEx	__REGBIT(0)
#define SYS_SMCSSMM_CSEx_DIS	__REGVAL(SYS_SMCSSMM_CSEx, 0)
#define SYS_SMCSSMM_CSEx_EN	__REGVAL(SYS_SMCSSMM_CSEx, 1)

/* General purpose, user-defined ID register */
#define SYS_GENID		__REG(0xa0900210)

/* External Interrupt x Control register */
#define SYS_EIC(x)		__REG2(0xa0900214, (x))

/* External Interrupt x Control register: Status */
#define SYS_EIC_STS		__REGBIT(3)

/* External Interrupt x Control register: Clear */
#define SYS_EIC_CLR		__REGBIT(2)

/* External Interrupt x Control register: Polarity */
#define SYS_EIC_PLTY		__REGBIT(1)
#define SYS_EIC_PLTY_AH		__REGVAL(SYS_EIC_PLTY, 0)
#define SYS_EIC_PLTY_AL		__REGVAL(SYS_EIC_PLTY, 1)

/* External Interrupt x Control register: Level edge */
#define SYS_EIC_LVEDG		__REGBIT(0)
#define SYS_EIC_LVEDG_LEVEL	__REGVAL(SYS_EIC_LVEDG, 0)
#define SYS_EIC_LVEDG_EDGE	__REGVAL(SYS_EIC_LVEDG, 1)

/* RTC Clock Control Register */
#define SYS_RTCCCR		__REG(0xa0900224)


#endif /* ifndef __ASM_ARCH_REGSSYS_H */
