/*
 * include/asm-arm/arch-ns9xxx/regs-usb.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#ifndef __ASM_ARCH_REGSUSB_H
#define __ASM_ARCH_REGSUSB_H

#include <asm/arch/hardware.h>


/* USB Module registers and addresses */
#define UHFE_BASE_PA		(0x90800000)
#define UHFE_BASE_VA		io_p2v(UHFE_BASE_PA)

#define UDFE_BASE_PA		(0x90900000)
#define UDFE_BASE_VA		io_p2v(UDFE_BASE_PA)

#define UDFE_DMA_BASE_PA	(0x90910000)
#define UDFE_DMA_BASE_VA	io_p2v(UDFE_DMA_BASE_PA)

#if defined(CONFIG_PROCESSOR_NS9360)
# define USB_MODULE_BASE_PA	(0x90801000)
# define USB_MODULE_BASE_VA	io_p2v(USB_MODULE_BASE_PA)
# define USB_REG_OFFSET		(0x0)
#elif defined(CONFIG_PROCESSOR_NS9750)
# define USB_MODULE_BASE_PA	(0x90100000)
# define USB_MODULE_BASE_VA	io_p2v(USB_MODULE_BASE_PA)
# define USB_REG_OFFSET		(0x1000)
#endif /* defined(CONFIG_PROCESSOR_NS9750) */

#define USB_DEVICE_PA		(UDFE_BASE_PA)
#define USB_DEVICE_PA_END	(UDFE_BASE_PA + 0x3fff)

#define USB_DEVICE_DMA_PA	(UDFE_DMA_BASE_PA)
#define USB_DEVICE_DMA_PA_END	(UDFE_DMA_BASE_PA + 0xfff)

/* the register addresses */
#define USB_GCTRL_STAT		__REG(USB_MODULE_BASE_PA)	/* NS9750 only */
#define USB_HCR			__REG(USB_MODULE_BASE_PA+USB_REG_OFFSET)
#define USB_IEN			__REG(USB_MODULE_BASE_PA+0x0c)
#define USB_ISTAT		__REG(USB_MODULE_BASE_PA+0x10)
#define USB_HCR_PSAT		__REG(USB_MODULE_BASE_PA+USB_REG_OFFSET+0x54)

#define UHFE_CTRL		__REG(0x90800000)
#define UHFE_IEN		__REG(0x9080000c)
#define UHFE_ISTAT		__REG(0x90800010)

/* register bit fields */
#define NS_USB_INT_GBL_EN 		(0x80000000)
#define NS_USB_INT_GBL_DMA 		(0x08000000)
#define NS_USB_INT_DMA_13 		(0x04000000)
#define NS_USB_INT_DMA_12		(0x02000000)
#define NS_USB_INT_DMA_11		(0x01000000)
#define NS_USB_INT_DMA_10		(0x00800000)
#define NS_USB_INT_DMA_9 		(0x00400000)
#define NS_USB_INT_DMA_8 		(0x00200000)
#define NS_USB_INT_DMA_7 		(0x00100000)
#define NS_USB_INT_DMA_6 		(0x00080000)
#define NS_USB_INT_DMA_5 		(0x00040000)
#define NS_USB_INT_DMA_4 		(0x00020000)
#define NS_USB_INT_DMA_3 		(0x00010000)
#define NS_USB_INT_DMA_2 		(0x00008000)
#define NS_USB_INT_DMA_1 		(0x00004000)
#define NS_USB_INT_FIFO	 		(0x00001000)
#define NS_USB_INT_URST	 		(0x00000800)
#define NS_USB_INT_SOF	 		(0x00000400)
#define NS_USB_INT_SUSPEND		(0x00000200)
#define NS_USB_INT_SETINF		(0x00000100)
#define NS_USB_INT_SETCFG		(0x00000080)
#define NS_USB_INT_WAKEUP		(0x00000040)
#define NS_USB_INT_OHCI_IRQ		(0x00000002)

/* the linux interrupt handler can only enable all DMA and all FIFO
 * interrupts. It is not yet possible to disable part of DMA's interrupts. */

#define NS_USB_INT_DMA_ALL		(NS_USB_INT_GBL_DMA | \
					 NS_USB_INT_DMA_13  | \
					 NS_USB_INT_DMA_12  | \
					 NS_USB_INT_DMA_11  | \
					 NS_USB_INT_DMA_10  | \
					 NS_USB_INT_DMA_9   | \
					 NS_USB_INT_DMA_8   | \
					 NS_USB_INT_DMA_7   | \
					 NS_USB_INT_DMA_6   | \
					 NS_USB_INT_DMA_5   | \
					 NS_USB_INT_DMA_4   | \
					 NS_USB_INT_DMA_3   | \
					 NS_USB_INT_DMA_2   | \
					 NS_USB_INT_DMA_1 )

#define NS_USB_INT_FIFO_ALL		(NS_USB_INT_FIFO    | \
					 NS_USB_INT_URST    | \
					 NS_USB_INT_SOF     | \
					 NS_USB_INT_SUSPEND | \
					 NS_USB_INT_SETINF  | \
					 NS_USB_INT_SETCFG  | \
					 NS_USB_INT_WAKEUP  | \
					 NS_USB_INT_OHCI_IRQ)

#define NS_USB_INT_HOST		( NS_USB_INT_OHCI_IRQ )

#define NS_UHFE_CTRL_WAKEUP	(0x00000004)
#define NS_UHFE_CTRL_HSTRST	(0x00000800)

#endif /* ifndef __ASM_ARCH_REGSUSB_H */
