/*
 * linux/include/asm-arm/arch-ns9xxx/system.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_SYSTEM_H
#define __ASM_ARCH_SYSTEM_H

#include <asm/proc-fns.h>
#include <asm/arch-ns9xxx/regs-sys.h>
#include <asm/mach-types.h>
#include <asm/arch-ns9xxx/regs-bbus.h>

#define CRYSTAL	294912
static inline u32 ns9xxx_systemclock(void)
{
	u32 pll = SYS_PLL;

	/* 
	 * This should be a multiple of HZ * TIMERCLOCKSELECT (in time.c)
	 */
	return 100 * CRYSTAL * (REGGET(pll, SYS_PLL, ND) + 1) /
		(1 << REGGET(pll, SYS_PLL, FS));
}

static inline u32 ns9xxx_cpuclock(void)
{
	return (ns9xxx_systemclock() / 2);
}

static inline u32 ns9xxx_ahbclock(void)
{
	return (ns9xxx_systemclock() / 4);
}

static inline u32 ns9xxx_bbusclock(void)
{
	return (ns9xxx_systemclock() / 8);
}

static inline void arch_idle(void)
{
	cpu_do_idle();
}

static inline void arch_reset(char mode)
{
	/* XXX: mode? */
	/* XXX: ??? */
	u32 reg;

	/* On CC9C, IO65 is used as BOOTMUX.  As SRESET doesn't reset the GPIO
	 * configurations, set IO65's to it's reset value (i.e. 0x3 = GPIO
	 * input) explicitly first. */
	if (machine_is_cc9c())
		(void)ns9xxx_gpio_cfgpin(65, GPIO_CFG_INPUT);
	
	reg = SYS_PLL >> 16;
	REGSET(reg, SYS_PLL, SWC, YES);
	SYS_PLL = reg;

	BUG();
}

#endif /* ifndef __ASM_ARCH_SYSTEM_H */
