/*
 * linux/include/asm-arm/arch-pxa/ccxp.h
 *
 * Support for the Digi ConnectCore XP Development Platform.
 *
 * @Author: Bernd Westermann
 * @References: [1] lubbock.h
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

/***********************************************************************
 *  @History:
 *     2005/06/27 : initial Version
 *     2005/09/19 : SL811 USB added
 ***********************************************************************/

#include <asm/arch/pxa-regs.h>
#include <asm/io.h>

#define PXA_PHYS_IO_BASE	0x40000000ul
#define PXA_PHYS_RAM		0xa0000000ul
#define PXA_BOOT_PARAMS		0xa0000100ul

/* Ethernet */
#define CCXP_ETH1_OFFSET	0x02000000 + 0x300
#define CCXP_ETH1_PHYS		PXA_CS1_PHYS + CCXP_ETH1_OFFSET
#define CCXP_ETH1_BASE		0xef000000ul
#define CCXP_ETH1_SIZE		0x00800000

/* SL811 USB */
#define CCXP_SL811_PHYS		0x31C00000
#define CCXP_SL811_BASE		0xf4000000
#define CCXP_SL811_DATA		0x31C00004
#define CCXP_SL811_SIZE		0x00000999

/*
 * USB SL811, PCMCIA and CF Interfaces
 */
#define CFG_MECR_VAL		0x00000003
#define CFG_MCMEM0_VAL		0x00040286
#define CFG_MCMEM1_VAL		0x00040286
#define CFG_MCATT0_VAL		0x00040286
#define CFG_MCATT1_VAL		0x00040286
#define CFG_MCIO0_VAL		0x00040286
#define CFG_MCIO1_VAL		0x00040286

static inline void gpio_set_bit(int gpio)
{
	GPSR(gpio) = GPIO_bit(gpio);
}
static inline void gpio_clr_bit(int gpio)
{
	GPCR(gpio) = GPIO_bit(gpio);
}
static inline int gpio_tst_bit(int gpio)
{
	return (GPLR(gpio) & GPIO_bit(gpio)) ? 1 : 0;
}

/* PCMCIA */
#define CCXP_PCMCIA_IO_0_BASE	(0xea000000ul)
#define CCXP_PCMCIA_IO_1_BASE	(0xec000000ul)
#define CCXP_PCMCIA_IO_0_MAPSIZE	0x02000000ul
#define CCXP_PCMCIA_IO_1_MAPSIZE	0x02000000ul
#define CCXP_PCMCIA_PHYS_IO_0_BASE	0x20000000ul
#define CCXP_PCMCIA_PHYS_IO_1_BASE	0x30000000ul
#define NUMBER_OF_CCXP_PCMCIA_SLOTS (2)
#define CCXP_CD1_GPIO		(82)
#define CCXP_RDY1_GPIO	(81)
#define CCXP_RST1_GPIO	(83)
#define CCXP_CD2_GPIO		(-1)
#define CCXP_RDY2_GPIO	(-1)
#define CCXP_RST2_GPIO	(-1)
// #define CCXP_BVD1_GPIO		(-1)
// #define CCXP_BVD2_GPIO		(-1)
#define VALID_SLOT(s)		(rdy_gpios[(s)] >= 0)

#define CCXP_IDE0_BASE		(CCXP_PCMCIA_IO_0_BASE + CCXP_IDE_OFFSET)
#define CCXP_IDE1_BASE		(CCXP_PCMCIA_IO_1_BASE + CCXP_IDE_OFFSET)
#define CCXP_OFFSET_IDE_CS1	0x00400000
#define CCXP_IDE0_CS1		(CCXP_IDE0_BASE + CCXP_OFFSET_IDE_CS1)
#define CCXP_IDE_OFFSET		0x00000000
#define CCXP_PHYS_IDE1_BASE	(PCMCIA_PHYS_IO_1_BASE + CCXP_IDE_OFFSET)
#define CCXP_OFFSET_IDE_CS0	0x00000000

#define HD_DRVADDR	0x3f7		/* Drive address register */
#define HD_IRQ_GPIO	(GPIO10_RTCCLK)
#define IRQ_HARDDISK	(IRQ_GPIO(HD_IRQ_GPIO))
#define PCMCIA_PHYS_IO_1_BASE	0x30000000ul

// // RTC DS1339
// #define CCXP_RTC_SCL_GPIO	8
// #define CCXP_RTC_SDA_GPIO	9
// #define DS1339_SCL_GPIO		CCXP_RTC_SCL_GPIO
// #define DS1339_SDA_GPIO		CCXP_RTC_SDA_GPIO

/* For XScale- MiniPCI interface */
#define CCXP_EXP_BASE            0x0C000000 /*0x14000000*/ 
#define CCXP_EXP_SIZE            0x2000
//
//#define REG8(addr)              (*(volatile u8 *)(addr))
#define REG16(addr)             (*(volatile u16 *)(addr))
#define REG32(addr)             (*(volatile u32 *)(addr))
//
/* For PCI support */
#define CCXP_EXP_PCI_MEM_BASE    CCXP_EXP_BASE + 0x00010000 /*0x14010000*/
#define CCXP_EXP_PCI_MEM_SIZE    0x00020000
#define CCXP_EXP_PCI_IO_BASE     CCXP_EXP_BASE + 0x00030000 /*0x14030000*/
#define CCXP_EXP_PCI_IO_SIZE     0x00010000 
//
/* 0x0000 - 0x03FF Register Space*/
//
//#define MST_EXP_PWR_CTRL        0x0000
#define CCXP_EXP_GEN_CTRL        0x0002
//#define MST_EXP_PCI_RD_UAB      0x0004
//#define MST_EXP_PCI_WR_UAB      0x0006
//#define MST_EXP_SRAM_RD_UAB     0x0008
//#define MST_EXP_SRAM_WR_UAB     0x000A
//#define MST_EXP_XMT_TGR         0x000C
//#define MST_EXP_RCV_TGR         0x000E
//#define MST_EXP_TEST_REG        0x0010
//
//#define PWR_CTRL_PCI_CLK_DOWN   (1<<6)
//#define PWR_CTRL_PCI_CLK_STAT   (1<<5)
//#define PWR_CTRL_FPGA_CLK_STAT  (1<<4)
//#define PWR_CTRL_FPGA_STATE_MSK 0x03 
//
//#define GEN_CTRL_RST            (1<<15)
//#define GEN_CTRL_PCI_INT_MSK    (1<<14)
//#define GEN_CTRL_TX_INT_MSK     (1<<13)
//#define GEN_CTRL_RX_INT_MSK     (1<<12)
//#define GEN_CTRL_PCI_INT_STAT   (1<<10)
//#define GEN_CTRL_TX_INT_STAT    (1<<9)
//#define GEN_CTRL_RX_INT_STAT    (1<<8)
#define GEN_CTRL_PCI_RD16_MODE  (1<<4)
//#define GEN_CTRL_PCI_CFG_MODE   (1<<0)    
//
/* 0x0400 - 0x07FF PCI Configuration Space */
#define CCXP_EXP_PCI_CONF_OFFSET   0x0400
//
/* 0x0800 - 0x0FFF PCI Memory Space */
//#define MST_EXP_PCI_MEM_OFFSET    0x0800
//
// /* 0x1000 - 0x17FF SRAM Buffer Space */
//#define MST_EXP_SRAM_BUF_OFFSET   0x1000
//
/* 0x1800 - 0x1FFF SRAM Descriptor Space */
//#define MST_EXP_SRAM_DESC_OFFSET  0x1800
//

//
//#define CCXP_PCMCIA0		CCXP_PCMCIA_IO_0_BASE
//#define CCXP_PCMCIA1		CCXP_PCMCIA_IO_1_BASE
//
//#define CCXP_PCMCIA_nIRQ		(1 << 10)  /* IRQ / ready signal */
//#define CCXP_PCMCIA_nSPKR_BVD2	(1 << 9)   /* VDD sense / digital speaker */
//#define CCXP_PCMCIA_nSTSCHG_BVD1	(1 << 8)   /* VDD sense / card status changed */
//#define CCXP_PCMCIA_nVS2		(1 << 7)   /* VSS voltage sense */
//#define CCXP_PCMCIA_nVS1		(1 << 6)   /* VSS voltage sense */
//#define CCXP_PCMCIA_nCD		(1 << 5)   /* Card detection signal */
//#define CCXP_PCMCIA_RESET	(1 << 4)   /* Card reset signal */
//#define CCXP_PCMCIA_PWR_MASK	(0x000f)   /* MAX1602 power-supply controls */
//
//#define CCXP_PCMCIA_PWR_VPP_0    0x0	   /* voltage VPP = 0V */
//#define CCXP_PCMCIA_PWR_VPP_120  0x2 	   /* voltage VPP = 12V*/
//#define CCXP_PCMCIA_PWR_VPP_VCC  0x1	   /* voltage VPP = VCC */
//#define CCXP_PCMCIA_PWR_VCC_0    0x0	   /* voltage VCC = 0V */
//#define CCXP_PCMCIA_PWR_VCC_33   0x8	   /* voltage VCC = 3.3V */
//#define CCXP_PCMCIA_PWR_VCC_50   0x4	   /* voltage VCC = 5.0V */

//#define CCXP_FPGA_PHYS	PXA_CS2_PHYS
//#define CCXP_FPGA_VIRT	(0xf0000000)
//#define CCXP_P2V(x)		((x) - CCXP_FPGA_PHYS + CCXP_FPGA_VIRT)
//#define CCXP_V2P(x)		((x) - CCXP_FPGA_VIRT + CCXP_FPGA_PHYS)

//#ifndef __ASSEMBLY__
//#  define __CCXP_REG(x)		(*((volatile unsigned long *)CCXP_P2V(x)))
//#else
//#  define __CCXP_REG(x)		CCXP_P2V(x)
//#endif

/* FPGA register virtual addresses */
//#define CCXP_WHOAMI		__CCXP_REG(CCXP_FPGA_PHYS + 0x000)
//#define CCXP_HEXLED		__CCXP_REG(CCXP_FPGA_PHYS + 0x010)
//#define CCXP_DISC_BLNK_LED	__CCXP_REG(CCXP_FPGA_PHYS + 0x040)
//#define CCXP_CONF_SWITCHES	__CCXP_REG(CCXP_FPGA_PHYS + 0x050)
//#define CCXP_USER_SWITCHES	__CCXP_REG(CCXP_FPGA_PHYS + 0x060)
//#define CCXP_MISC_WR		__CCXP_REG(CCXP_FPGA_PHYS + 0x080)
//#define CCXP_MISC_RD		__CCXP_REG(CCXP_FPGA_PHYS + 0x090)
//#define CCXP_IRQ_MASK_EN		__CCXP_REG(CCXP_FPGA_PHYS + 0x0c0)
//#define CCXP_IRQ_SET_CLR		__CCXP_REG(CCXP_FPGA_PHYS + 0x0d0)
//#define CCXP_GP			__CCXP_REG(CCXP_FPGA_PHYS + 0x100)

//#ifndef __ASSEMBLY__
//extern void ccxp_set_misc_wr(unsigned int mask, unsigned int set);
//#endif
